#!/usr/bin/perl -w

# the above assumes that the perl program is in /usr/bin; if necessary,
# change to suit your system

#################### START POD DOCUMENTATION ##############

=head1 Main

Package 'Main'

=head2 NAME

Parsing the PAML Files

=head2 DESCRIPTION

This is the main program which presents an output of tolerated and non-tolerated amino acids based on PAML output.

=head2 FEEDBACK

All feedback (bugs, feature enhancements, etc.) are greatly appreciated. 

=head2 AUTHOR

Danny W. De Kee (dan.dekee@gmail.com)

=head2 METHODS

=cut

###### Here is the Start of the main package #######
package main;  ## Start of Main Package
#use Data::Dumper;
use lib '../DETECTER';
use Node;
use Site;
use NodesBlock;

my $nodes_block = new NodesBlock();
if ($ARGV[0]){
	$nodes_block->parse_file("$ARGV[0]"); #parsing the file
}else {
	$dir = "../data";
	$nodes_block->parse_file("$dir/ABC.Redo.CFTRsubtree.Crop.txt"); #parsing the file
}

my $extant_char;
my $site;

for (my $i = 0; $i < $nodes_block->get_numb_of_nodes; $i++) {  ## Reading all nodes
	my $node = $nodes_block->get_node($i);
	for (my $j = 0; $j < $node->get_numb_of_sites; $j++) {      ## Reading all sites within the nodes
		my $site = $node->get_site($j);	
		my $site_id = $site->get_site_id;	
		$extant_char{$site_id} = $site->{uniq_seq} if $i==0;
		my $prob = $site->{probability};
		foreach my $aa (keys %{$prob}){
			if ($prob->{$aa} >= 0.050) {
				$extant_char{$site_id} .= $aa;
			}
		}
	}
}
foreach my $site (sort keys %extant_char) {
	$tempsite = new Site;  ## Dummy Object in order to remove duplicates from extant characters
	$extant_char{$site} =$tempsite->process_sequence($extant_char{$site});
	my   $AAList = "acdefghiklmnpqrstvwy";
	foreach $extantchar (split(//,$extant_char{$site}) ) { $AAList =~ s/$extantchar/ /gi; }
	printf("%-6s %-20s : %s\n",$site,$extant_char{$site},$AAList);
}
### End of the program
